@@ -44,8 +44,6 @@ sub empty {
     return $buffer;
 }
 
-sub get { substr shift->{buffer}, 0, shift }
-
 sub get_line {
     my $self = shift;
 
@@ -126,10 +124,6 @@ the following new ones.
 
     my $chunk = $buffer->empty;
 
-=head2 C<get>
-
-    my $chunk = $buffer->get(4);
-
 =head2 C<get_line>
 
    my $line = $buffer->get_line;
